import {
    generateGlobalColorCss,
    generatePalette,
    generateTextAlignCss,
    createNuskinTheme,
    iterateNuskinThemeColors,
} from '../theme-utils';

const validTheme = {
    palette: {
        primary: {
            main: '#252525',
            contrastText: '#FFFFFF',
            buttonState: {
                hovered: '#8C8C8C',
                disabled: '#E0E0E0',
                pressed: '#F5F5F5',
                focused: '#252525',
                disabledText: '#8C8C8C',
            },
            darkMode: {
                main: '#F5F5F5',
                contrastText: '#252525',
                buttonState: {
                    hovered: '#C66464',
                    hoveredText: '#FFFFFF',
                    pressed: '#A54553',
                    pressedText: '#FFFFFF',
                    focused: '#F5F5F5',
                    disabled: '#5F5F5F',
                },
            },
        },
    },
};

describe('Theme Utils', () => {
    it('1. generatePalette to return light mode values', () => {
        const palette = generatePalette(validTheme.palette, false);
        expect(palette?.primary?.main).toBe('#252525');
    });

    it('2. generatePalette to return dark mode values', () => {
        const palette = generatePalette(validTheme.palette, true);
        expect(palette?.primary?.main).toBe('#F5F5F5');
    });

    it('3. generatePalette to return {} palette is undefined', () => {
        const palette = generatePalette(undefined, true);
        expect(palette).toStrictEqual({});
    });

    it('4. generatePalette to return {} palette is null', () => {
        const palette = generatePalette(undefined, true);
        expect(palette).toStrictEqual({});
    });

    it('5. should return all colors when prefersDarkMode is false', () => {
        const result = generatePalette(validTheme.palette, false);
        expect(result).toEqual(validTheme.palette);
    });

    it('6. should return all colors when prefersDarkMode is not passed', () => {
        const result = generatePalette(validTheme.palette);
        expect(result).toEqual(validTheme.palette);
    });

    it('7. generateGlobalColorCss to return the proper css string', () => {
        const cssString = generateGlobalColorCss(validTheme);
        expect(cssString).toContain('.bg-primary');
        expect(cssString).toContain('.text-primary');
        expect(cssString).toContain('.border-primary');
    });

    it('8. generateGlobalColorCss to return the proper css string', () => {
        const cssString = generateGlobalColorCss(validTheme);
        expect(cssString).toContain('.bg-primary');
        expect(cssString).toContain('.text-primary');
        expect(cssString).toContain('.border-primary');
    });

    it('9. generateGlobalColorCss to have the .dark class', () => {
        const theme = {
            ...validTheme,
            colors: {
                accentColorA: {
                    main: '#F56767',
                    dark: '#FC9492',
                },
            },
        };
        const cssString = generateGlobalColorCss(theme);
        expect(cssString).toContain('.dark {');
    });

    it('10. generateGlobalColorCss to not have the .dark class', () => {
        const cssString = generateGlobalColorCss(validTheme);
        expect(cssString).not.toContain('.dark {');
    });

    it('11. generateGlobalColorCss to return empty string if theme is falsy', () => {
        const cssString = generateGlobalColorCss(null as any);
        expect(cssString).toBe('');
    });

    it('12. generateTextAlignCss to output all text align classes', () => {
        const cssString = generateTextAlignCss();
        expect(cssString).toContain('.text-left');
        expect(cssString).toContain('.text-right');
        expect(cssString).toContain('.text-center');
        expect(cssString).toContain('.text-justify');
        expect(cssString).toContain('.text-start');
        expect(cssString).toContain('.text-end');
    });

    it('13. createNuskinTheme to inherit from MUI Theme', () => {
        const theme = createNuskinTheme(validTheme, true);
        expect(theme?.palette?.primary?.main).toBe('#252525');
        expect(theme?.palette?.mode).toBe('dark');
    });

    it('14. createNuskinTheme to inherit from MUI Theme', () => {
        const theme = createNuskinTheme();
        expect(theme?.palette?.primary?.main).toBe('#252525');
        expect(theme?.palette?.mode).toBe('light');
    });

    it('15. when nothing is passed to iterateNuskinThemeColors, an empty object is returned', () => {
        const result = iterateNuskinThemeColors();
        expect(result).toEqual({});
    });
});
