import { createTheme, Palette, PaletteColor, PaletteKeys } from '@mui/material/styles';

import nuskinTheme from '../themes/genome-foundation/';

import { FoundationTheme } from '../themes';
import { isValidPaletteColor } from './common-utils';

export const generatePalette = (colors?: Palette, prefersDarkMode = false) => {
    if (typeof colors !== 'object' || !colors) {
        return {};
    }
    const allColorKeys = Object.keys(colors) as FoundationTheme.PaletteColors[];
    const colorKeys = allColorKeys?.filter((colorKey) => isValidPaletteColor(colorKey));
    return colorKeys.reduce((result, paletteName: PaletteKeys) => {
        const colorObject = colors?.[paletteName] as PaletteColor;
        if (prefersDarkMode) {
            const darkMode = colorObject?.darkMode;
            if (darkMode) {
                result[paletteName] = darkMode;
            }
        } else if (colorObject) {
            result[paletteName] = colorObject;
        }
        return result;
    }, {} as Record<string, PaletteColor>);
};

export const createNuskinTheme = (theme: FoundationTheme.Theme = nuskinTheme, prefersDarkMode?: boolean) => {
    const muiTheme = createTheme({ ...theme });
    const mode: FoundationTheme.Mode = prefersDarkMode ? 'dark' : 'light';
    return {
        ...muiTheme,
        palette: {
            ...muiTheme.palette,
            ...theme?.palette,
            mode,
        },
    };
};

export const iterateNuskinThemeColors = (colorObj: FoundationTheme.Colors | Record<string, string> = {}) => {
    const allThemeColors: Record<string, string | FoundationTheme.MainDarkColors> = {};

    const iterate = (
        colorObj: FoundationTheme.Colors | Record<string, string | FoundationTheme.MainDarkColors>,
        parentColor?: string
    ) => {
        Object.keys(colorObj).forEach((key) => {
            const colorKey = parentColor ? `${parentColor}-${key}` : key;
            const colorValue = colorObj[key as FoundationTheme.PaletteColors];
            if (typeof colorValue === 'string') {
                allThemeColors[colorKey] = colorValue;
            } else {
                if (colorValue?.main) {
                    allThemeColors[colorKey] = colorValue as FoundationTheme.MainDarkColors;
                } else if (typeof colorValue === 'object') {
                    iterate(colorValue, colorKey);
                }
            }
        });
    };

    iterate(colorObj);
    return allThemeColors;
};

const generateColors = (name: string, value: string | { main?: string; dark?: string }) => {
    const makeColorCss = (colorValue: string) => `
        .bg-${name} {
            background: ${colorValue};
        }
        .text-${name} {
            color: ${colorValue};
        }
        .border-${name} {
            border-color: ${colorValue};
        }
    `;
    if (typeof value !== 'string' && value?.main) {
        let cssText = makeColorCss(value.main);
        if (value?.dark) {
            cssText += `
                .dark {
                    ${makeColorCss(value?.dark)}
                }
            `;
        }
        return cssText;
    }
    return makeColorCss(value as string);
};

export const generateGlobalColorCss = (theme: FoundationTheme.Theme) => {
    if (!theme) {
        return '';
    }

    let colorCssString = '';
    /*
     * Nuskin Theme consists of palette, which is sent to Mui
     * The Nuskin Theme also has colors which is used to generate the helper css
     */
    const palette = theme?.palette;

    const themeColors = iterateNuskinThemeColors(theme?.colors);

    const paletteColorNames = Object.keys(palette || {}) as FoundationTheme.PaletteColors[];

    const paletteColors = paletteColorNames?.filter((color: FoundationTheme.PaletteColors) => {
        const colorInPalette = palette?.[color] as FoundationTheme.MainDarkColors;
        return colorInPalette?.main;
    });

    colorCssString = paletteColors?.reduce((result, colorName) => {
        const colorResult = palette?.[colorName] as FoundationTheme.MainDarkColors;
        const colorValue = colorResult?.main;
        if (colorValue) {
            result += generateColors(colorName, colorValue);
        }
        return result;
    }, '');

    colorCssString = Object.keys(themeColors)?.reduce((result, colorName: string) => {
        const colorValue = themeColors?.[colorName];
        if (colorValue) {
            result += generateColors(colorName, colorValue);
        }
        return result;
    }, colorCssString);

    return colorCssString;
};

export const generateTextAlignCss = () => {
    return `
        .text-left {
            text-align: left;
        }
        .text-center {
            text-align: center;
        }
        .text-right {
            text-align: right;
        }
        .text-justify {
            text-align: justify;
        }
        .text-start {
            text-align: start;
        }
        .text-end {
            text-align: end;
        }
    `;
};
